#!/usr/bin/python
import numpy as np
import sys
import os
import shutil

##############################################################################################################
#
# 7.2015: CC: Cleaned, polished, rewritten
#
# 6.2015: AB: Adopted from BASH to PYTHON from A_band_gap_at_different_volumes.bash.
#
##############################################################################################################
# Script computes the band gap at different temperatures by using the temperature dependent
#   volumes/lattice expansion that need to be provided in file: T_a0_alpha.dat
#  --> See ex. 3 for the generation of T_a0_alpha.dat
#
# List of temperatures can be changed by changing min_T,max_T,step_T in main()
#
# Output written to band_gap.dat
#
##############################################################################################################

# Write fcc geometry of Si to geometry.in-type file
def write_geometry(lat_par, filename="geometry.in"):
  f = open(filename, "w")
  f.write('%s %3.3f %13.9f %13.9f\n' %('lattice_vector', 0, lat_par, lat_par ))
  f.write('%s %13.9f %3.3f %13.9f\n' %('lattice_vector', lat_par, 0, lat_par ))
  f.write('%s %13.9f %13.9f %3.3f\n' %('lattice_vector', lat_par, lat_par, 0 ))
  f.write('%s %3.3f %3.3f %3.3f %s\n' %('atom', 0 , 0 , 0, 'Si' ))
  f.write('%s %13.9f %13.9f %13.9f %s\n' %('atom', lat_par/2, lat_par/2, lat_par/2, 'Si' ))
  f.close()

# Extract band gap from FHI-aims output:
def get_bandgap_from_output(filename ="aims.out"):
  out_file = open(filename, "r")
  for line in out_file:
    if (line.find('| Energy difference      : ')>=0):
      E_gap=float(line.split()[4])
  out_file.close()
  return E_gap

# Get temperature dependent lattice constants from lattice expansion file
def get_lattice_parameter_from_lattice_expansion( temperature, filename='T_a0_alpha.dat' ):
  # Initialize:
  lattice_para = np.zeros_like( temperature , dtype='float64' )
  current_index = 0
  current_T     = format(temperature[current_index], '.3f')
  # Go over individual lines:
  alpha_file   = open(filename, "r")
  for line in alpha_file:
    line=line.strip()
    if ( (not line.startswith("#")) and line.startswith(current_T) ):
      lattice_para[current_index] = float(line.split()[1])
      current_index = current_index + 1
      if ( current_index >= len(temperature) ):
        break
      current_T     = format(temperature[current_index], '.3f')
  alpha_file.close()
  return lattice_para

# Generate directory tree and run calculations:
def run_loop_over_lattice_constants( temperature , lattice_para ):
  # Initialize
  bandgaps = np.zeros_like( lattice_para )
  # Run loop over temperatures/lattice constants:
  for current_index,current_T in enumerate(temperature):
    current_directory='T_'+format( current_T , '.3f')
    if ( os.path.exists( current_directory ) == False): 
      print("  | %8.3f K : Runnning band structure calculation for a lattice constant of %9.5f AA" % ( current_T, lattice_para[current_index] ))
      os.mkdir (current_directory)
      os.chdir (current_directory)
      write_geometry( lattice_para[current_index] , filename="geometry.in")
      shutil.copy('../control.in', '.' )
      os.system('mpirun -np 4 aims.x > aims.out ')
    else:
      print("  | %8.3f K : Precomputed FHI-aims output found: Skipping!" % ( current_T ))
      os.chdir (current_directory )
    bandgaps[current_index] = get_bandgap_from_output("aims.out")
    os.chdir ('../')
  return bandgaps

# Simple output at the end:
def write_output( temperature, lattice_para, bandgaps, filename="band_gap.dat"):
  output= open(filename, "w")
  output.write('%s \n' %( '# Computing band gap shift due to lattice expansion at different temperatures:') )
  output.write('#   T (K)       a (AA)     Band gap (eV)\n')
  for T,a,bg in zip(temperature,lattice_para,bandgaps):
    output.write('%9.3f %13.7f %13.7f  \n' % ( T, a, bg ))
  output.close()
  print("  | Finished: Succesfully written file '%s' .\n" % ( filename ))

###################################################################################################################################
## Here starts the main part:
def main():
  # Greet the user and inform him what is going on:
  print("\n ------ Calculating the Volume(Temperature)-dependent Band Gap using the Lattice Expansion Coefficient -------\n ")
  
  # Check that file exists:
  file_lattice_expansion='T_a0_alpha.dat'
  if (os.path.exists(file_lattice_expansion) == False):
    sys.exit( "\n....ERROR: '%s' file not found  \n" % ( file_lattice_expansion ) ) 
  else:
    print("- Step A: File '%s' found.\n" % ( file_lattice_expansion ))
  
  # Generate T - V(T)
  print("- Step B: Following temperatures and lattice constants (volumes) will be considered:")
  min_T  = 0
  max_T  = 1010
  step_T = 50
  temperatures = np.array( list(range(min_T,max_T,step_T)) )
  lattice_para = get_lattice_parameter_from_lattice_expansion( temperatures, filename=file_lattice_expansion )
  for T,a in zip( temperatures,lattice_para):
    print("  | %8.3f K --> %9.5f AA" % ( T,a ))
  
  # Build directory tree and perform calculations
  print("\n- Step C: Performing band structure calculation for different temperatures / volumes V(T)")
  bandgaps = run_loop_over_lattice_constants( temperatures , lattice_para )
  
  # Ouput
  print("\n- Step D: Writing Volume(Temperature)-dependent Band Gap to file 'band_gap.dat'.")
  write_output( temperatures , lattice_para , bandgaps, "band_gap.dat")

if __name__ == "__main__":
    main()
